@@ -15,10 +15,12 @@ import os
 from botocore import xform_name
 from botocore.docs.bcdoc.docevents import DOC_EVENTS
 from botocore.model import StringShape
+from botocore.utils import is_json_value_header
 
 from awscli import SCALAR_TYPES
 from awscli.argprocess import ParamShorthandDocGen
 from awscli.topictags import TopicTagDB
+from awscli.utils import find_service_and_method_in_event_name
 
 LOG = logging.getLogger(__name__)
 
@@ -42,6 +44,11 @@ class CLIDocumentEventHandler(object):
     def build_translation_map(self):
         return dict()
 
+    def _get_argument_type_name(self, shape, default):
+        if is_json_value_header(shape):
+            return 'JSON'
+        return default
+
     def _map_handlers(self, session, event_class, mapfn):
         for event in DOC_EVENTS:
             event_handler_name = event.replace('-', '_')
@@ -127,7 +134,8 @@ class CLIDocumentEventHandler(object):
             self._documented_arg_groups.append(argument.group_name)
         else:
             option_str = '%s <value>' % argument.cli_name
-        if not argument.required:
+        if not (argument.required
+                or getattr(argument, '_DOCUMENT_AS_REQUIRED', False)):
             option_str = '[%s]' % option_str
         doc.writeln('%s' % option_str)
 
@@ -158,7 +166,8 @@ class CLIDocumentEventHandler(object):
             self._documented_arg_groups.append(argument.group_name)
         else:
             name = '``%s``' % argument.cli_name
-        doc.write('%s (%s)\n' % (name, argument.cli_type_name))
+        doc.write('%s (%s)\n' % (name, self._get_argument_type_name(
+            argument.argument_model, argument.cli_type_name)))
         doc.style.indent()
         doc.include_doc_string(argument.documentation)
         self._document_enums(argument, doc)
@@ -298,12 +307,29 @@ class ServiceDocumentEventHandler(CLIDocumentEventHandler):
 
 class OperationDocumentEventHandler(CLIDocumentEventHandler):
 
+    AWS_DOC_BASE = 'https://docs.aws.amazon.com/goto/WebAPI'
+
     def build_translation_map(self):
         operation_model = self.help_command.obj
         d = {}
         for cli_name, cli_argument in self.help_command.arg_table.items():
             if cli_argument.argument_model is not None:
-                d[cli_argument.argument_model.name] = cli_name
+                argument_name = cli_argument.argument_model.name
+                if argument_name in d:
+                    previous_mapping = d[argument_name]
+                    # If the argument name is a boolean argument, we want the
+                    # the translation to default to the one that does not start
+                    # with --no-. So we check if the cli parameter currently
+                    # being used starts with no- and if stripping off the no-
+                    # results in the new proposed cli argument name. If it
+                    # does, we assume we have the postive form of the argument
+                    # which is the name we want to use in doc translations.
+                    if cli_argument.cli_type_name == 'boolean' and \
+                            previous_mapping.startswith('no-') and \
+                            cli_name == previous_mapping[3:]:
+                        d[argument_name] = cli_name
+                else:
+                    d[argument_name] = cli_name
         for operation_name in operation_model.service_model.operation_names:
             d[operation_name] = xform_name(operation_name, '-')
         return d
@@ -313,6 +339,24 @@ class OperationDocumentEventHandler(CLIDocumentEventHandler):
         operation_model = help_command.obj
         doc.style.h2('Description')
         doc.include_doc_string(operation_model.documentation)
+        self._add_webapi_crosslink(help_command)
+
+    def _add_webapi_crosslink(self, help_command):
+        doc = help_command.doc
+        operation_model = help_command.obj
+        service_model = operation_model.service_model
+        service_uid = service_model.metadata.get('uid')
+        if service_uid is None:
+            # If there's no service_uid in the model, we can't
+            # be certain if the generated cross link will work
+            # so we don't generate any crosslink info.
+            return
+        doc.style.new_paragraph()
+        doc.write("See also: ")
+        link = '%s/%s/%s' % (self.AWS_DOC_BASE, service_uid,
+                             operation_model.name)
+        doc.style.external_link(title="AWS API Documentation", link=link)
+        doc.writeln('')
 
     def _json_example_value_name(self, argument_model, include_enum_values=True):
         # If include_enum_values is True, then the valid enum values
@@ -402,7 +446,9 @@ class OperationDocumentEventHandler(CLIDocumentEventHandler):
                 doc.style.new_line()
         doc.write('}')
 
-    def doc_option_example(self, arg_name, help_command, **kwargs):
+    def doc_option_example(self, arg_name, help_command, event_name, **kwargs):
+        service_name, operation_name = \
+            find_service_and_method_in_event_name(event_name)
         doc = help_command.doc
         cli_argument = help_command.arg_table[arg_name]
         if cli_argument.group_name in self._arg_groups:
@@ -414,7 +460,7 @@ class OperationDocumentEventHandler(CLIDocumentEventHandler):
         docgen = ParamShorthandDocGen()
         if docgen.supports_shorthand(cli_argument.argument_model):
             example_shorthand_syntax = docgen.generate_shorthand_example(
-                cli_argument.cli_name, cli_argument.argument_model)
+                cli_argument, service_name, operation_name)
             if example_shorthand_syntax is None:
                 # If the shorthand syntax returns a value of None,
                 # this indicates to us that there is no example
@@ -492,7 +538,8 @@ class OperationDocumentEventHandler(CLIDocumentEventHandler):
     def _do_doc_member_for_output(self, doc, member_name, member_shape, stack):
         docs = member_shape.documentation
         if member_name:
-            doc.write('%s -> (%s)' % (member_name, member_shape.type_name))
+            doc.write('%s -> (%s)' % (member_name, self._get_argument_type_name(
+                                      member_shape, member_shape.type_name)))
         else:
             doc.write('(%s)' % member_shape.type_name)
         doc.style.indent()
